/**
 * @file artis/kernel/Context.hpp
 * @author See the AUTHORS file
 */

/*
 * Copyright (C) 2012-2017 ULCO http://www.univ-littoral.fr
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __ARTIS_KERNEL_CONTEXT_HPP
#define __ARTIS_KERNEL_CONTEXT_HPP

#include <artis/kernel/State.hpp>

#include <boost/serialization/serialization.hpp>

namespace artis { namespace kernel {

template < typename U >
class Context
{
public:
    Context()
    { }

    Context(double begin, double end) : _begin(begin), _end(end)
    { }

    virtual ~Context()
    { }

    double begin() const
    { return _begin; }

    double end() const
    { return _end; }

    const State < U >& state() const
    { return _state; }

    State < U >& state()
    { return _state; }

private:
    friend class boost::serialization::access;

    template < class Archive >
    void serialize(Archive & ar, const unsigned int version)
    {
        (void) version;

        ar & _begin;
        ar & _end;
        ar & _state;
    }

    double      _begin;
    double      _end;
    State < U > _state;
};

} }

#endif
